import random

import requests
from django.conf import settings
from faker import Faker
from .base import BaseIdentityService, LookupResponse


class MockIdentityService(BaseIdentityService):
    def lookup_bvn(self, bvn):
        fake = Faker()
        return LookupResponse(
            first_name=fake.first_name(),
            last_name=fake.last_name(),
            middle_name=fake.name_female(),
            date_of_birth=str(fake.date_of_birth()),
            gender=random.choice(['Male', 'Female']),
            photo='',
            phone_number=str(fake.phone_number()),
            match=bool(random.randint(0, 1)),
            confidence_value='90.000'

        )

    def lookup_nin(self, nin):
        fake = Faker()
        return LookupResponse(
            first_name=fake.first_name(),
            last_name=fake.last_name(),
            middle_name=fake.name_female(),
            date_of_birth=str(fake.date_of_birth()),
            gender=random.choice(['Male', 'Female']),
            photo='',
            phone_number=str(fake.phone_number()),
            match=bool(random.randint(0, 1)),
            confidence_value='80.000'

        )

    def lookup_bvn_with_image(self, bvn, image):
        fake = Faker()
        confidence_value = round(random.uniform(10.0, 100.0), 2)

        return LookupResponse(
            first_name=fake.first_name(),
            last_name=fake.last_name(),
            middle_name=fake.name_female(),
            date_of_birth=str(fake.date_of_birth()),
            gender=random.choice(['Male', 'Female']),
            photo=image,
            phone_number=str(fake.phone_number()),
            match=bool(confidence_value > 70.0),
            confidence_value=str(confidence_value)

        )



from datetime import datetime
import os

from .base import BaseRequest


class AuthService(BaseRequest):
    base_url = None

    def __init__(self, request):
        super().__init__(request)
        base_url = f"{os.getenv('AUTH_SERVICE_BASE_URL', None)}/api/v1"
        self.base_url = f"{base_url}"

    def get_user_by_id(self, id):
        path = f'users/{id}'
        return self.send_request("GET", path)


def get_auth_token(request=None):

    if request is None:
        return {}
   
    request_time=datetime.now()
    request_time = request_time.strftime("%Y-%m-%dT%H:%M:%S.%f%z")
    if isinstance(request, dict):
        auth_token = request['headers']["Authorization"]
    else:
       auth_token = request.headers.get("Authorization")
    auth_token = auth_token.split()[1] if "Bearer" in auth_token else auth_token
    return {"Authorization": f"Bearer {auth_token}"}



import json
import time

from django.conf import settings
from django.urls import Resolver404, resolve
from django.utils import timezone

from .tasks import send_logs_to_elk
from .contrib import get_headers, get_client_ip, mask_sensitive_data, decode_jwt_token


class APILoggerMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

        # Initialize settings from Django settings or use defaults
        self.API_LOGGER_PATH_TYPE = getattr(settings, 'API_LOGGER_PATH_TYPE', 'ABSOLUTE')
        if self.API_LOGGER_PATH_TYPE not in ['ABSOLUTE', 'RAW_URI', 'FULL_PATH']:
            self.API_LOGGER_PATH_TYPE = 'ABSOLUTE'

        self.API_LOGGER_SKIP_URL_NAME = getattr(settings, 'API_LOGGER_SKIP_URL_NAME', [''])
        if not isinstance(self.API_LOGGER_SKIP_URL_NAME, (tuple, list)):
            self.API_LOGGER_SKIP_URL_NAME = ['']

        self.API_LOGGER_SKIP_NAMESPACE = getattr(settings, 'API_LOGGER_SKIP_NAMESPACE', [])
        if not isinstance(self.API_LOGGER_SKIP_NAMESPACE, (tuple, list)):
            self.API_LOGGER_SKIP_NAMESPACE = []

        self.API_LOGGER_CONTENT_TYPES = [
            "application/json",
            "application/vnd.api+json",
            "application/gzip",
            "application/octet-stream",
        ]

        self.API_LOGGER_EXCLUDE_HTTP_METHODS = ['GET']
        if hasattr(settings, 'API_LOGGER_EXCLUDE_HTTP_METHODS'):
            if type(settings.API_LOGGER_EXCLUDE_HTTP_METHODS) is tuple or type(
                    settings.API_LOGGER_EXCLUDE_HTTP_METHODS) is list:
                self.API_LOGGER_EXCLUDE_HTTP_METHODS = [
                    item.upper() for item in settings.API_LOGGER_EXCLUDE_HTTP_METHODS]


    def __call__(self, request):
        try:
            if hasattr(settings, 'ELK_LOGGER_URL'):
                method = request.method
                # Resolve URL name and namespace
                try:
                    resolver_match = resolve(request.path_info)
                    url_name = resolver_match.url_name
                    namespace = resolver_match.namespace
                except Resolver404:
                    url_name = None
                    namespace = None

                # Always skip logging for requests in 'admin' namespace
                if namespace == 'admin':
                    return self.get_response(request)

                # Skip logging based on configured URL names
                if url_name in self.API_LOGGER_SKIP_URL_NAME:
                    return self.get_response(request)

                # Skip logging based on configured namespaces
                if namespace in self.API_LOGGER_SKIP_NAMESPACE:
                    return self.get_response(request)

                if len(self.API_LOGGER_EXCLUDE_HTTP_METHODS) > 0 and method in self.API_LOGGER_EXCLUDE_HTTP_METHODS:
                    return self.get_response(request)

                # Measure request execution time
                start_time = time.time()

                # Fetch request headers and method
                headers = get_headers(request=request)
                method = request.method

                # Parse request body based on content type
                request_data = {}
                if request.content_type == "application/json":
                    try:
                        request_data = json.loads(request.body)
                    except json.JSONDecodeError:
                        request_data = request.body.decode('utf-8')
                elif request.content_type in ["multipart/form-data", "application/x-www-form-urlencoded"]:
                    request_data = request.POST.dict()
                    if request.FILES:
                        request_data['files'] = {key: file.name for key, file in request.FILES.items()}

                email, user_id = decode_jwt_token(request)

                response = self.get_response(request)

                # Determine response body based on content type
                if response.get("content-type") in self.API_LOGGER_CONTENT_TYPES:
                    if response.get('content-type') == 'application/gzip':
                        response_body = '** GZIP Archive **'
                    elif response.get('content-type') == 'application/octet-stream':
                        response_body = '** Binary File **'
                    elif getattr(response, 'streaming', False):
                        response_body = '** Streaming **'
                    else:
                        if isinstance(response.content, bytes):
                            response_body = json.loads(response.content.decode())
                        else:
                            response_body = json.loads(response.content)

                    # Determine API path based on configuration
                    if self.API_LOGGER_PATH_TYPE == 'ABSOLUTE':
                        api = request.build_absolute_uri()
                    elif self.API_LOGGER_PATH_TYPE == 'FULL_PATH':
                        api = request.get_full_path()
                    elif self.API_LOGGER_PATH_TYPE == 'RAW_URI':
                        api = request.get_raw_uri()
                    else:
                        api = request.build_absolute_uri()

                    # Mask sensitive data in the logged data
                    data = dict(
                        api=mask_sensitive_data(api, mask_api_parameters=True),
                        headers=mask_sensitive_data(headers),
                        body=mask_sensitive_data(request_data),
                        method=method,
                        client_ip_address=get_client_ip(request),
                        response=mask_sensitive_data(response_body),
                        status_code=response.status_code,
                        execution_time=time.time() - start_time,
                        added_on=timezone.now(),
                        email=email,
                        user_id=user_id,
                        environment=settings.ENVIRONMENT_INSTANCE,
                    )

                    # Convert certain fields to JSON for logging purposes
                    d = data.copy()
                    d['headers'] = json.dumps(d['headers'], indent=4, ensure_ascii=False) if d.get('headers') else ''
                    if request_data:
                        d['body'] = json.dumps(d['body'], indent=4, ensure_ascii=False) if d.get('body') else ''
                    d['response'] = json.dumps(d['response'], indent=4, ensure_ascii=False) if d.get('response') else ''
                    send_logs_to_elk.delay(d)

            else:
                response = self.get_response(request)
            return response

        except Exception as e:
            print(e)
            return self.get_response(request)




from rest_framework.exceptions import PermissionDenied, AuthenticationFailed


def get_user_permissions(user):
    """
    Fetch all permissions a user has
    """
    if user.is_authenticated:
        return user.permissions
    raise AuthenticationFailed


def check_user_has_permissions(user, perms):
    """
    Function to check if a user has any of the permissions passed.
    """
    user_permissions = get_user_permissions(user)

    def check_perm(perm_list):
        return any(_perm in perm_list for _perm in perms)

    if user.is_admin is False and perms and check_perm(user_permissions) is False:
        raise PermissionDenied

class PermissionMixin:
    """
    Custom Permission mixin
    """
    custom_permissions = None

    def check_permissions(self, request):
        check_user_has_permissions(request.user, self.get_custom_permissions())
        return super().check_permissions(request)

    def get_custom_permissions(self):
        return self.custom_permissions


class TenantMixin:
    def get_queryset(self, *args, **kwargs):
        return self.queryset.filter(tenant_id=self.request.user.tenant_id)
